@@ -8,6 +8,15 @@ from copy import deepcopy
 from dataclasses import dataclass, asdict
 from steam_nb_api.ledet.ParametersLEDET import ParametersLEDET
 
+@dataclass
+class QuenchSweep:
+    VarToQuench: str = ""
+    iStartQuench: np.ndarray = np.array([])
+    ValToQuench: np.ndarray = np.array([])
+    QuenchCopy: np.array = np.array([])
+
+PossibleQuenchVar = ["tStartQuench", "lengthHotSpot_iStartQuench", "vQ_iStartQuench"]
+
 class ParameterSweep(object):
     def __init__(self, ParametersLEDET):
         self.ParametersLEDET = ParametersLEDET
@@ -15,6 +24,7 @@ class ParameterSweep(object):
         self.ParameterMatrix = np.array([])
         self.SweepMatrix = np.array([])
         self.VoidRatio = np.array([])
+        self.QuenchSweep = QuenchSweep()
 
     def _addToParameterMatrix(self, values):
         if(self.ParameterMatrix.size == 0):
@@ -83,11 +93,23 @@ class ParameterSweep(object):
     def _generateImitate(self, classvalue, value):
         if type(classvalue) == np.ndarray:
             v = deepcopy(classvalue)
-            v.fill(value)
+            v = np.where(self.ParametersLEDET.Inputs.polarities_inGroup != 0, value, v)
         else:
             v = value
         return v
 
+    def _generateImitate_Quench(self, classvalue, value):
+        if type(classvalue) == np.ndarray:
+            v = deepcopy(self.QuenchSweep.QuenchCopy)
+            if len(self.QuenchSweep.iStartQuench.shape) > 1:
+                for i in range(self.QuenchSweep.iStartQuench.shape[1]):
+                    v[self.QuenchSweep.iStartQuench[int(value) -1, i]-1] = self.QuenchSweep.ValToQuench[int(value) - 1, i]
+                    v[self.QuenchSweep.iStartQuench[int(value) -1, i]-1] = self.QuenchSweep.ValToQuench[int(value) - 1, i]
+            else:
+              v[self.QuenchSweep.iStartQuench[int(value)-1]-1] = self.QuenchSweep.ValToQuench[int(value)-2]
+              v[self.QuenchSweep.iStartQuench[int(value)-1]-1] = self.QuenchSweep.ValToQuench[int(value)-2]
+        return v
+
     def loadSweepMatrixFromExcel(self, file):
         self.__cleanSweepMatrix()
         self.__cleanParametersToSweep()
@@ -116,6 +138,8 @@ class ParameterSweep(object):
         if cleanFolder:
             filelist = [f for f in os.listdir(folder)]
             for f in filelist:
+                if "selfMutualInductanceMatrix" in f:
+                    continue
                 os.remove(os.path.join(folder, f))
         #2. Generate Permutations
         self.generatePermutations()
@@ -126,13 +150,21 @@ class ParameterSweep(object):
             file_name = folder + file_name_stub + "_" + str(i+OffsetNumber) +".xlsx"
             for j in range(self.SweepMatrix.shape[1]):
                 SetClass = self._checkSetClass(self.ParametersToSweep[j])
-                setValue = self._generateImitate(self.ParametersLEDET.getAttribute(getattr(self.ParametersLEDET, SetClass), self.ParametersToSweep[j]), self.SweepMatrix[i,j])
+                if self.ParametersToSweep[j] in PossibleQuenchVar:
+                    if self.QuenchSweep.QuenchCopy.size == 0:
+                        self.QuenchSweep.QuenchCopy = deepcopy(self.ParametersLEDET.getAttribute(getattr(self.ParametersLEDET, SetClass), self.ParametersToSweep[j]))
+                    setValue = self._generateImitate_Quench(self.ParametersLEDET.getAttribute(getattr(self.ParametersLEDET, SetClass), self.ParametersToSweep[j]), self.SweepMatrix[i,j])
+                else:
+                    setValue = self._generateImitate(self.ParametersLEDET.getAttribute(getattr(self.ParametersLEDET, SetClass), self.ParametersToSweep[j]), self.SweepMatrix[i,j])
                 self.ParametersLEDET.setAttribute(getattr(self.ParametersLEDET, SetClass), self.ParametersToSweep[j], setValue)
                 if (self.ParametersToSweep[j] == "overwrite_f_internalVoids_inGroup"):
                     setVal = self.VoidRatio - self.SweepMatrix[i, j]
-                    # exValue = self.VoidRatio-self.SweepMatrix[i, j]
-                    # setVal = self._generateImitate(self.ParametersLEDET.getAttribute(getattr(self.ParametersLEDET, "Inputs"), "overwrite_f_externalVoids_inGroup"), exValue)
+                    if any(sV < 0 for sV in setVal):
+                        print("Negative externalVoids calculated. Abort Sweep, please check.")
+                        return
+                    setVal = np.where(self.ParametersLEDET.Inputs.polarities_inGroup != 0, setVal, 0)
                     self.ParametersLEDET.setAttribute(getattr(self.ParametersLEDET, "Inputs"), "overwrite_f_externalVoids_inGroup", setVal)
+
             self.ParametersLEDET.writeFileLEDET(file_name)
 
         #4. Write CSV with all Parameters
@@ -177,7 +209,7 @@ class MinMaxSweep(ParameterSweep):
 
     def _activateHelium(self):
         setValue = self._generateImitate(self.ParametersLEDET.getAttribute(getattr(self.ParametersLEDET, "Inputs"),
-                                                                           "nStrands_inGroup"), 0)
+                                                                           "polarities_inGroup"), 0)
         self.ParametersLEDET.setAttribute(getattr(self.ParametersLEDET, "Inputs"), "overwrite_f_internalVoids_inGroup", setValue)
         self.ParametersLEDET.setAttribute(getattr(self.ParametersLEDET, "Inputs"), "overwrite_f_externalVoids_inGroup", setValue)
 
@@ -192,3 +224,18 @@ class MinMaxSweep(ParameterSweep):
         ins_total = (cs_ins - cs_bare)/cs_ins
         self.VoidRatio = (cs_bare - cs_strand)/cs_ins
         self.addParameterToSweep("overwrite_f_internalVoids_inGroup", minHe/100.0, maxHe/100.0, basePoints = basePoints)
+
+    def addQuenchSweep(self, VarToQuench, iStartQuench, ValToQuench):
+        iStartQuench = np.array(iStartQuench)
+        ValToQuench = np.array(ValToQuench)
+        if not iStartQuench.shape == ValToQuench.shape:
+            print("Variable To Sweep and provided Values do not have same shape. Abort.")
+        self.QuenchSweep.VarToQuench = np.array(VarToQuench)
+        self.QuenchSweep.iStartQuench = np.array(iStartQuench)
+        self.QuenchSweep.ValToQuench = np.array(ValToQuench)
+        self.ParametersToSweep.append(VarToQuench)
+        try:
+            imitate = np.linspace(1, len(ValToQuench), len(ValToQuench))
+        except:
+            imitate = np.array([0])
+        self._addToParameterMatrix(np.array([imitate]))
