@@ -14,7 +14,7 @@ from steam_nb_api.ledet.ParameterSweep import *
 from steam_nb_api.ledet.ParametersLEDET import ParametersLEDET
 import csv
 
-def EvaluateSimulations(MatFolder, MagnetName, MeasFile, SweepObject, Mat = True, Plots = True, SkipAlign = False, SetdITol = 200):
+def EvaluateSimulations(MatFolder, MagnetName, MeasFile, SweepObject, Mat = True, Plots = True, SkipAlign = False, SetdITol = 200, showBestFit = 10):
     print("Reading Data and evaluate")
     #Collect Simulated Data
     flag_BasePoints = 0
@@ -52,6 +52,11 @@ def EvaluateSimulations(MatFolder, MagnetName, MeasFile, SweepObject, Mat = True
                         if ".sys" not in item:
                             num = item.replace(MagnetName + '_VariableHistory_', '')
                             num = num.replace(".txt", '')
+                            try:
+                                x = int(num)
+                            except:
+                                print("Magnetname does not fit to file names. Please check!")
+                                return
                             with open(MatFolder+"//" + item, 'r') as file:
                                 k = file.readline().split(',')[:-1]
                                 try:
@@ -64,14 +69,14 @@ def EvaluateSimulations(MatFolder, MagnetName, MeasFile, SweepObject, Mat = True
                                 try:
                                     idxR = k.index(' R_CoilSections_1')
                                     R = np.array(cols[idxR])
-                                    T = np.array(cols[idxT])
+                                    T  = np.array(cols[idxT])
                                     RLedet = np.append([num], np.array(R))
                                     if RData.size == 0:
                                         RData = RLedet.astype(float)
                                     elif not T.size == SimData[0, :].size:
                                         RL = interp1d(T, np.array(RLedet[1:]), kind='cubic')
                                         RLedet = RL(SimData[0, :])
-                                    RData = np.vstack((RData, RLedet.astype(float)))
+                                        RData = np.vstack((RData, RLedet.astype(float)))
                                 except:
                                     print("Error reading R, no abort")
                                     pass
@@ -158,12 +163,12 @@ def EvaluateSimulations(MatFolder, MagnetName, MeasFile, SweepObject, Mat = True
     print("Best Fit for Meas-StartIDX: ", idx_low)
     BestFit = np.argsort(MSESims[:, 1])
     print("Best fitting simulations for MSE: ")
-    print(MSESims[BestFit[:10], 0])
-    print(MSESims[BestFit[:10], 1])
+    print(MSESims[BestFit[:showBestFit], 0])
+    print(MSESims[BestFit[:showBestFit], 1])
     BestFit = np.argsort(LDSims[:, 1])
     print("Best fitting simulations based on smallest, largest Deviation: ")
-    print(LDSims[BestFit[:10], 0])
-    print(LDSims[BestFit[:10], 1])
+    print(LDSims[BestFit[:showBestFit], 0])
+    print(LDSims[BestFit[:showBestFit], 1])
 
     SweepMatrix = SweepObject.SweepMatrix
     ParametersToSweep = SweepObject.ParametersToSweep
@@ -171,6 +176,7 @@ def EvaluateSimulations(MatFolder, MagnetName, MeasFile, SweepObject, Mat = True
     #Parameter Importance Study -- Ablation Analysis
     # 1. Extract best 10% of the Simulations
     NumberBest = int(np.floor(SimData.shape[0]/10))
+    if SimData.shape[0]<10: NumberBest = SimData.shape[0]-3
     BestFit = np.argsort(MSESims[:, 1])[0:NumberBest]
     SweepedParameters = SweepMatrix.shape[1]
     AbMatrix = np.zeros((SweepedParameters, NumberBest))
@@ -194,18 +200,22 @@ def EvaluateSimulations(MatFolder, MagnetName, MeasFile, SweepObject, Mat = True
 
     #Make fancy plot
     if Plots:
-        #Plot two most important parameters together on MSE
-        Vals1 = SweepObject.ParameterMatrix[Order[0], :][SweepObject.ParameterMatrix[Order[0], :] != 0]
-        Vals2 = SweepObject.ParameterMatrix[Order[1], :][SweepObject.ParameterMatrix[Order[1], :] != 0]
+        flag_No3Dplots = 0
         try:
+            #Plot two most important parameters together on MSE
+            Vals1 = SweepObject.ParameterMatrix[Order[0], :][SweepObject.ParameterMatrix[Order[0], :] != 0]
+            Vals2 = SweepObject.ParameterMatrix[Order[1], :][SweepObject.ParameterMatrix[Order[1], :] != 0]
             Vals3 = SweepObject.ParameterMatrix[Order[2], :][SweepObject.ParameterMatrix[Order[2], :] != 0]
         except:
             print("No 3D Data Found. Following Plots are not plottable.")
-            return
+            flag_No3Dplots = 1
         InitVals = SweepMatrix[0, :]
         D1 = np.zeros((Vals1.shape[0]))
-        D2 = np.zeros((Vals2.shape[0]))
-        D3 = np.zeros((Vals3.shape[0]))
+        try:
+            D2 = np.zeros((Vals2.shape[0]))
+            D3 = np.zeros((Vals3.shape[0]))
+        except:
+            pass
         for i in range(Vals1.shape[0]):
             for j in range(SweepMatrix.shape[0]):
                 if SweepMatrix[j, Order[0]] == Vals1[i]:
@@ -215,28 +225,34 @@ def EvaluateSimulations(MatFolder, MagnetName, MeasFile, SweepObject, Mat = True
                         Idx = MSESims[:, 0]
                         Idx = Idx[Idx == j]
                         D1[i] = int(Idx[0])
-        for i in range(Vals2.shape[0]):
-            for j in range(SweepMatrix.shape[0]):
-                if SweepMatrix[j, Order[1]] == Vals2[i]:
-                    Cp = np.delete(SweepMatrix[j, :], Order[1])
-                    Cp2 = np.delete(InitVals[:], Order[1])
-                    if np.allclose(Cp, Cp2):
-                        Idx = MSESims[:, 0]
-                        Idx = Idx[Idx == j]
-                        D2[i] = int(Idx[0])
-        for i in range(Vals3.shape[0]):
-            for j in range(SweepMatrix.shape[0]):
-                if SweepMatrix[j, Order[2]] == Vals3[i]:
-                    Cp = np.delete(SweepMatrix[j, :], Order[2])
-                    Cp2 = np.delete(InitVals[:], Order[2])
-                    if np.allclose(Cp, Cp2):
-                        Idx = MSESims[:, 0]
-                        Idx = Idx[Idx == j]
-                        D3[i] = int(Idx[0])
+        try:
+            for i in range(Vals2.shape[0]):
+                for j in range(SweepMatrix.shape[0]):
+                    if SweepMatrix[j, Order[1]] == Vals2[i]:
+                        Cp = np.delete(SweepMatrix[j, :], Order[1])
+                        Cp2 = np.delete(InitVals[:], Order[1])
+                        if np.allclose(Cp, Cp2):
+                            Idx = MSESims[:, 0]
+                            Idx = Idx[Idx == j]
+                            D2[i] = int(Idx[0])
+            for i in range(Vals3.shape[0]):
+                for j in range(SweepMatrix.shape[0]):
+                    if SweepMatrix[j, Order[2]] == Vals3[i]:
+                        Cp = np.delete(SweepMatrix[j, :], Order[2])
+                        Cp2 = np.delete(InitVals[:], Order[2])
+                        if np.allclose(Cp, Cp2):
+                            Idx = MSESims[:, 0]
+                            Idx = Idx[Idx == j]
+                            D3[i] = int(Idx[0])
+        except:
+            pass
 
         x = SweepMatrix[D1.astype(int), Order[0]]
-        y = SweepMatrix[D2.astype(int), Order[1]]
-        k = SweepMatrix[D3.astype(int), Order[2]]
+        try:
+            y = SweepMatrix[D2.astype(int), Order[1]]
+            k = SweepMatrix[D3.astype(int), Order[2]]
+        except:
+            pass
 
         ##Plot1
         fig = plt.figure()
@@ -265,6 +281,7 @@ def EvaluateSimulations(MatFolder, MagnetName, MeasFile, SweepObject, Mat = True
         ax.legend(fontsize =11)
         ax.grid(True)
 
+        if flag_No3Dplots==1: return
         ##Plot 2
         verts = []
         zs = [1.0, 2.0, 3.0]
