@@ -1,5 +1,8 @@
 import os
 import json
+import pandas as pd
+import numpy as np
+import shutil
 
 from steam_nb_api.resources.ResourceReader import ResourceReader
 from steam_nb_api.utils.misc import makeCopyFile
@@ -380,6 +383,8 @@ class ParametersCOSIM:
         for n in range(0, N_LEDET):
             if DistinctMagnets != 1:
                 DM = n+1
+            else:
+                DM = 1
             for i in range(len(CoilSections)):  # Number of Ports
                 for j in range(2):  # 2Ports in PSpice
                     struct = {}
@@ -422,3 +427,83 @@ class ParametersCOSIM:
             for i in range(len(struct_total)):
                 json.dump(struct_total[i], f, indent=4)
                 f.write('\n')
+
+    def copyCOSIMfiles(self, nameFolderCosimModel, nameFileSING, StimulusFile, LEDETFiles, nameMagnet,N_LEDET=1):
+        # Copy PSPICE model file
+        nameFolderPSpiceModel = os.path.join(nameFolderCosimModel, 'PSpice')
+        if not os.path.isdir(nameFolderPSpiceModel):
+            os.mkdir(nameFolderPSpiceModel)
+        makeCopyFile(nameFileSING, os.path.join(nameFolderPSpiceModel, 'Circuit.cir'))
+        makeCopyFile(StimulusFile, os.path.join(nameFolderPSpiceModel, 'ExternalStimulus.stl'))
+
+        LEDET_Fs = []
+        for i in range(N_LEDET):
+            # Copy LEDET model files - MCD
+            if N_LEDET > 1:
+                nameFolderLedetModel = os.path.join(nameFolderCosimModel, 'LEDET_'+str(i))
+                sourcedir = LEDETFiles[i]
+                sourcefiles = os.listdir(LEDETFiles[i])
+                nM = nameMagnet[i]
+            else:
+                nameFolderLedetModel = os.path.join(nameFolderCosimModel, 'LEDET')
+                sourcefiles = os.listdir(LEDETFiles)
+                nM = nameMagnet
+                sourcedir = LEDETFiles
+            if not os.path.isdir(nameFolderLedetModel):
+                os.mkdir(nameFolderLedetModel)
+            destinationpath_field = nameFolderLedetModel + "//Field maps//" + nM
+            destinationpath_para = nameFolderLedetModel + "//LEDET//" + nM + "//Input//"
+            for file in sourcefiles:
+                if file.endswith('.map2d') and not file.startswith(".sys"):
+                    makeCopyFile(os.path.join(sourcedir, file), os.path.join(destinationpath_field, file))
+                if file.endswith('.xlsx') and not file.startswith(".sys") and file.startswith(str(nM)):
+                    shutil.copy(os.path.join(sourcedir, file), os.path.join(destinationpath_para, file))
+                    LEDET_Fs.append(file)
+                if file.endswith('_selfMutualInductanceMatrix.csv'):
+                    shutil.copy(os.path.join(sourcedir, file), os.path.join(destinationpath_para, file))
+
+        return LEDET_Fs
+
+    def prepareLEDETFiles(self, files, N_PAR):
+        TurnNumber = 0
+
+        if type(files) == list:
+            for i in range(len(files)):
+                file = files[i]
+                Tnew = self.prepareSingleLEDETFile(file)
+                TurnNumber = TurnNumber + Tnew
+            TurnNumber = int(TurnNumber / len(files))
+        else:
+            TurnNumber= self.prepareSingleLEDETFile(files)
+
+        ## Do some kind of check
+        TurnNumber_All = TurnNumber*N_PAR
+
+        if TurnNumber_All >= 10000:
+            print("You are trying to simulate ",str(TurnNumber_All),"in paralell mode. \n")
+            maxN_PAR = np.floor(10000/TurnNumber)
+            print("Consider not to user more than ", str(maxN_PAR)," parallel Simulations.")
+
+    def prepareSingleLEDETFile(self, file):
+        df_full = pd.read_excel(file, sheet_name=None, header=None)
+        df = df_full["Options"]
+        df.rename(columns={0: 'a', 1: 'b', 2: 'c'}, inplace=True)
+        df.loc[df['b'] == 'flag_generateReport', 'c'] = 0
+        df.loc[df['b'] == 'flag_saveMatFile', 'c'] = 0
+        df_full["Options"] = df
+
+        df = df_full['Inputs']
+        df.rename(columns={0: 'a', 1: 'b', 2: 'c'}, inplace=True)
+        x = df.loc[df['b'] == 'HalfTurnToInductanceBlock']
+        x = x.iloc[0]
+        TurnNumber = max(x[2:-1])
+
+        writer = pd.ExcelWriter(file)
+        df_full['Inputs'].to_excel(writer, index=False, index_label=False, header=False, sheet_name='Inputs')
+        df_full['Options'].to_excel(writer, index=False, index_label=False, header=False, sheet_name='Options')
+        df_full['Plots'].to_excel(writer, index=False, index_label=False, header=False, sheet_name='Plots')
+        df_full['Variables'].to_excel(writer, index=False, index_label=False, header=False,
+                                      sheet_name='Variables')
+        writer.save()
+
+        return TurnNumber
